from impacket.dcerpc.v5 import transport, epm
from impacket.dcerpc.v5.transport import DCERPCStringBinding

from impacket import uuid
from impacket.dcerpc.v5 import mgmt
from impacket.dcerpc.v5.rpcrt import DCERPCException, RPC_C_AUTHN_LEVEL_PKT_PRIVACY


class getUUIDs:
    def __init__(self, stringbinding=''):
        try:
            # __stringbinding = objekt triedy impacket.dcerpc.v5.transport.DCERPCStringBinding
            self.__stringbinding = DCERPCStringBinding(stringbinding)
        except Exception as _e:
            raise Exception("Nesprávny obsah stringbinding!")
        # __rpctransport = objekt triedy impacket.dcerpc.v5.transport.TCPTransport
        self.__rpctransport = transport.DCERPCTransportFactory(stringbinding)
        # __dce = objekt triedy impacket.dcerpc.v5.rpcrt.DCERPC_v5
        self.__dcerpc = self.__rpctransport.get_dce_rpc()

    # vola metodu impacket.dcerpc.v5.rpcrt.DCERPC_v5.set_credentials()
    def set_rpc_credentials(self, username, password, domain='', hashes=None):
        if hasattr(self.__dcerpc, 'set_credentials'):
            self.__dcerpc.set_credentials(username, password, domain, '', '')

    # vola metodu impacket.dcerpc.v5.transport.TCPTransport.connect()
    def connect(self):
        self.__dcerpc.set_auth_level(RPC_C_AUTHN_LEVEL_PKT_PRIVACY)
        self.__dcerpc.connect()

    def execute(self):
        try:
            # vola metodu impacket.dcerpc.v5.rpcrt.DCERPC_v5.bind()
            # spojenie s Remote Management Interface - UUID(AFA8BD80-7D8A-11C9-BEF4-08002B102989, Ver1)
            self.__dcerpc.bind(mgmt.MSRPC_UUID_MGMT)
            # ziskanie Interface IDs (UUIDs) z Remote Management Interface
            # vola metodu impacket.dcerpc.v5.rpcrt.DCERPC.request()
            ifids = mgmt.hinq_if_ids(self.__dcerpc)
            uuids = set(
                uuid.bin_to_uuidtup(ifids['if_id_vector']['if_id'][i]['Data'].getData())
                for i in range(ifids['if_id_vector']['count'])
            )
            # pridanie MSRPC_UUID_MGMT
            uuids.add(('AFA8BD80-7D8A-11C9-BEF4-08002B102989', '1.0'))
            # vypis ziskanych UUIDs
            for u in sorted(uuids):
                if u[0] in epm.KNOWN_PROTOCOLS:
                    print('Protokol: %s' % (epm.KNOWN_PROTOCOLS[u[0]]))
                else:
                    print('Protokol: neznámy')
                if uuid.uuidtup_to_bin(u)[:18] in epm.KNOWN_UUIDS:
                    print('Služba: %s' % (epm.KNOWN_UUIDS[uuid.uuidtup_to_bin(u)[:18]]))
                else:
                    print('Služba: neznáma')
                print('UUID: %s v%s' % (u[0], u[1]))
                print('------------------------------------------------------------------------')
        except DCERPCException as _e:
            if str(_e).find('nca_s_unk_if') >= 0 or \
                    str(_e).find('reason_not_specified') >= 0 or \
                    str(_e).find('abstract_syntax_not_supported') >= 0:
                print("Cieľové Remote Management Interface je nedostupné")
            elif str(_e).find('rpc_s_access_denied') is False:
                print("Cieľové Remote Management Interface vyžaduje autentifikáciu")
            else:
                raise

    # vola metodu impacket.dcerpc.v5.transport.TCPTransport.disconnect()
    def disconnect(self):
        self.__dcerpc.disconnect()


if __name__ == '__main__':
    gUUIDs = None
    try:
        # ncacn_ip_tcp patri do Microsoft RPC protocol sequences
        # https://learn.microsoft.com/en-us/windows/win32/rpc/selecting-a-protocol-sequence
        # UUID = Universally Unique IDentifier
        # https://datatracker.ietf.org/doc/html/rfc4122
        gUUIDs = getUUIDs('ncacn_ip_tcp:192.168.1.62[135]')
        # vola metodu impacket.dcerpc.v5.rpcrt.DCERPC_v5.set_credentials()
        gUUIDs.set_rpc_credentials('fero', 'fero')
        # vola metodu impacket.dcerpc.v5.transport.TCPTransport.connect()
        gUUIDs.connect()
        # ziskanie UUIDs z Remote Management Interface
        gUUIDs.execute()
        # vola metodu impacket.dcerpc.v5.transport.TCPTransport.disconnect()
        gUUIDs.disconnect()
    except Exception as e:
        print('Chyba: %s' % e)
